import React, { Component } from 'react';
import { createNavigationContainer } from 'react-navigation';
import WebViewStackRouter from '../routers/webview-stack-router';
import WebViewActions from '../routers/webview-actions';
import CardStackTransitioner from '../views/card-stack-transitioner';

export default (routeConfigs, navigatorConfig = {}) => {
  const {
    initialRouteName,
    initialRouteParams,
    paths,
    headerMode,
    mode,
    cardStyle,
    transitionConfig,
    onTransitionStart,
    onTransitionEnd,
    navigationOptions,
    webViewConfig,
  } = navigatorConfig;
  const stackRouterConfig = {
    initialRouteName,
    initialRouteParams,
    paths,
    navigationOptions,
    webViewConfig,
  };

  const navigator = ((View) => {
    const router = WebViewStackRouter(routeConfigs, stackRouterConfig);

    class Navigator extends Component {
      static router = router;
      static routeConfigs = routeConfigs;
      static navigatorConfig = navigatorConfig;
      static navigatorType = null;

      constructor(props) {
        super(props);

        this.state = {
          webViewBackPromiseHandled: false,
        };
      }

      props;

      componentWillReceiveProps(nextProps) {
        if (
          nextProps.navigation.state.webViewBackPromise &&
          !this.state.webViewBackPromiseHandled
        ) {
          this.setState({ webViewBackPromiseHandled: true });
          nextProps.navigation.state.webViewBackPromise.then(() => {
            this.setState({ webViewBackPromiseHandled: false });
            this.props.navigation.dispatch(WebViewActions.clearBackPromise({
              // key: this.props.scene.key.replace('scene_', ''),

              // FIXME: we are not specifying a key here, so this will cancel all ViewBackPromises for all VWSNs in the app
            }));
            console.log('clearBackPromise');
          });
        }
      }

      render() {
        return <View {...this.props} router={router} />;
      }
    }

    return Navigator;
  })(props => (
    <CardStackTransitioner
      {...props}
      headerMode={headerMode}
      mode={mode}
      cardStyle={cardStyle}
      transitionConfig={transitionConfig}
      onTransitionStart={onTransitionStart}
      onTransitionEnd={onTransitionEnd}
    />
  ));


  return createNavigationContainer(navigator, navigatorConfig.containerOptions);
};
