import { StackRouter } from 'react-navigation';
import { NavigationActions } from 'react-navigation';
import { get, omit } from 'lodash';
import WebViewActions from './webview-actions';

export default (routeConfigs, stackConfig) => {
  const stackRouter = StackRouter(routeConfigs, stackConfig);

  return {
    ...stackRouter,
    stackConfig,
    routeConfigs,
    getStateForAction(action, state) {
      if (WebViewActions.isWebViewAction(action)) {

        // skip if we're recieving an action for another vwsn
        if(action.key) {
          if (!state.routes.some(r => r.key === action.key)) {
            return state;
          }
        }

        if (action.type === WebViewActions.REF) {
          return {
            ...state,
            webViewLoad: action.ref.load,
            webViewReplace: action.ref.replace,
            webViewReload: action.ref.reload,
            webViewBack: action.ref.goBack,
            webViewOnGoBack: action.ref.onGoBack,
            webViewExecute: action.ref.executeFunction,
          };
        }

        if (action.type === WebViewActions.LOAD) {
          state.webViewLoad(action.uri);
          return state;
        }

        if (action.type === WebViewActions.REPLACE) {
          state.webViewReplace(action.uri);
          return state;
        }

        if (action.type === WebViewActions.RELOAD) {
          state.webViewReload();
          return state;
        }

        if (action.type === WebViewActions.LOADING) {
          const setParamsAction = NavigationActions.setParams({
            params: { loading: action.loading },
            key: state.routes[state.index].key,
          });

          return stackRouter.getStateForAction(setParamsAction, state);
        }

        if (action.type === WebViewActions.CLEAR_BACK_PROMISE) {
          // console.log('INDEX:', state.index);
          return omit(state, 'webViewBackPromise');
        }
      }

      if (action.type === 'Navigation/BATCH') {
        return action.actions.reduce((prevState, currAction) => (
          stackRouter.getStateForAction(currAction, prevState)
        ), state);
      }

      if (action.type === NavigationActions.BACK) {
        const currIsWebView = get(state.routes, `[${state.index}].params.webView`, false);
        const prevWasWebView = get(state.routes, `[${state.index - 1}].params.webView`, false);

        // console.log('INDEX:', state.index);
        // console.log('currIsWebView:', currIsWebView);
        // console.log('prevWasWebView:', prevWasWebView);

        if (
          currIsWebView &&
          prevWasWebView &&
          !state.webViewBackPromise
        ) {
          if (state.webViewOnGoBack) {
            state.webViewOnGoBack();
          }

          const newState = stackRouter.getStateForAction(NavigationActions.back(), state);
          console.log(newState.routes[newState.index].key);
          return stackRouter.getStateForAction(NavigationActions.setParams({
            params: { loading: true },
            key: newState.routes[newState.index].key,
          }), { ...newState, webViewBackPromise: state.webViewBack() });
        }

        if (state.webViewBackPromise) {
          return null;
        }

        return stackRouter.getStateForAction(action, state);
      }

      return stackRouter.getStateForAction(action, state);
    },
  };
};
