import React, { PureComponent, PropTypes } from 'react';
import { View, ActivityIndicator, Image } from 'react-native';
import { get } from 'lodash';
import styles from './webview-screen-styles';

const propTypes = {
  navigation: PropTypes.object.isRequired,
  screenProps: PropTypes.object,
};

const defaultProps = {
  screenProps: {},
};

const navigationOptions = {
  header: null,
};

class WebViewScreen extends PureComponent {
  constructor(props) {
    super(props);

    this.getParam = (path, defaultValue) => (
      get(this.props.navigation.state, `params.${path}`, defaultValue)
    );
    this.getScreenProp = (path, defaultValue) => (
      get(this.props.screenProps, path, defaultValue)
    );
  }

  setWebViewLayout = ({ nativeEvent }) => {
    const { width, height, x, y } = nativeEvent.layout;

    if (
      this.props.navigation.state.params &&
      (
        this.props.navigation.state.params.webViewWidth !== width ||
        this.props.navigation.state.params.webViewHeight !== height ||
        this.props.navigation.state.params.webViewX !== x ||
        this.props.navigation.state.params.webViewY !== y
      )
    ) {
      this.props.navigation.setParams({
        webViewWidth: width,
        webViewHeight: height,
        webViewX: x,
        webViewY: y,
      });
    }
  }

  renderSpinner() {
    const spinnerBackroundColor = this.getScreenProp('loadingOverlay.backgroundColor', '#fff');
    const activityIndicatorColor = this.getScreenProp('loadingOverlay.activityIndicatorColor', null);

    return (
      <View style={[styles.spinner, { backgroundColor: spinnerBackroundColor }]}>
        <ActivityIndicator color={activityIndicatorColor} />
      </View>
    );
  }

  render() {
    const image = this.getParam('image', null);
    const loading = this.getParam('loading', true);
    const Component = image ? Image : View;

    return (
      <View style={styles.container} >
        <Component
          onLayout={this.setWebViewLayout}
          style={styles.placeholder}
          {...(image ? { source: { uri: image } } : {})}
        >
          { loading ? this.renderSpinner() : null }
        </Component>
      </View>
    );
  }
}

WebViewScreen.propTypes = propTypes;
WebViewScreen.defaultProps = defaultProps;
WebViewScreen.navigationOptions = navigationOptions;

export default WebViewScreen;
