import React from 'react';
import { Platform } from 'react-native';
import { get } from 'lodash';
import ReactNavigationHeader from 'react-navigation/src/views/Header';
import HeaderBackButton from 'react-navigation/src/views/HeaderBackButton';

const APPBAR_HEIGHT = Platform.OS === 'ios' ? 44 : 56;
const STATUSBAR_HEIGHT = Platform.OS === 'ios' ? 20 : 0;
const TITLE_OFFSET = Platform.OS === 'ios' ? 70 : 40;

class Header extends ReactNavigationHeader {
  _renderLeftComponent = (props) => {
    const options = this.props.getScreenDetails(props.scene).options;
    if (typeof options.headerLeft !== 'undefined') {
      return options.headerLeft;
    }
    if (props.scene.index === 0) {
      return null;
    }
    const backButtonTitle = this._getBackButtonTitleString(props.scene);
    const truncatedBackButtonTitle = this._getTruncatedBackButtonTitle(
      props.scene,
    );
    const width = this.state.widths[props.scene.key]
      ? (this.props.layout.initWidth - this.state.widths[props.scene.key]) / 2
      : undefined;
    const onPress = () => {
      if (
        get(props.scene.route, 'params.webView', false) &&
        get(this.props.scenes[props.scene.index - 1].route, 'params.webView', false)
      ) {
        this.props.webView.goBack();
      } else {
        this.props.navigation.goBack(null);
      }
    };
    return (
      <HeaderBackButton
        onPress={onPress}
        pressColorAndroid={options.headerPressColorAndroid}
        tintColor={options.headerTintColor}
        title={backButtonTitle}
        truncatedTitle={truncatedBackButtonTitle}
        titleStyle={options.headerBackTitleStyle}
        width={width}
      />
    );
  };
}

export default Header;
