import {
  Platform,
} from 'react-native';
import { NavigationActions } from 'react-navigation';
import { takeSnapshot } from 'react-native-view-shot';
import { isUndefined, get } from 'lodash';

const navigationTypes = {
  load: -1,
  linkClicked: 0,
  formSubmitted: 1,
  backForward: 2,
  reload: 3,
  other: 5,
};

let initialized;
let onLoadingStart;

function setParams(params, key) {
  const action = NavigationActions.setParams({
    params,
    ...(key ? { key: key.replace('scene_', '') } : {}),
  });

  this.props.navigation.dispatch(action);
}

// function navigate(routeName, params) {
//   const action = NavigationActions.navigate({ routeName, params });
//
//   this.props.navigation.dispatch(action);
// }

function getHeaderHeight(webView, headerSelector) {
  const fn = (selector) => {
    if (!selector) {
      return document.elementFromPoint(1, 1).getBoundingClientRect().bottom;
    }

    return document.querySelector(selector).getBoundingClientRect().bottom - 2; // FIXME
  };

  return webView.executeFunction(fn, [headerSelector]);
}

function onNavigationStateChange(event, scene) {
  const { url, loading, canGoBack } = event;

  if(Platform.OS === 'android' && !this.pageLoading) {
    onLoadingStart = false;
    initialized = true;
  }

  if (iOSLoading(event)) {
    const fakeHeader = get(this.props.screenProps, 'fakeHeader', false);
    const headerSelector = get(this.props.screenProps, 'headerSelector', null);

    // const action = createNavigateAction(scene, this.props.router);
    // this.props.navigation.dispatch(action);

    const promises = [takeSnapshot(this.webView, { format: 'jpeg', quality: 0.8 })];
    if (fakeHeader) {
      promises.push(getHeaderHeight(this.webView, headerSelector));
    }

    Promise.all(promises).then(([snapshot, headerHeight]) => {
      // const params = {
      //   image: snapshot,
      //   ...(fakeHeader && !isUndefined(headerHeight) ? { headerHeight } : {}),
      // };
      // setParams.apply(this, [params, scene.key]);

      this.props.navigation.dispatch({
        type: 'Navigation/BATCH',
        actions: [
          NavigationActions.setParams({
            key: scene.key.replace('scene_', ''),
            params: {
              loading: false,
              hideWebView: true,
              image: snapshot,
              ...(fakeHeader && !isUndefined(headerHeight) ? { headerHeight } : {}),
            },
          }),
          createNavigateAction(scene, this.props.router),
        ],
      });
    });
  } else if (androidLoading(this.pageLoading, event)) {
    onLoadingStart = true;
    const action = createNavigateAction(scene, this.props.router);
    this.props.navigation.dispatch(action);
  } else {
    setParams.apply(this, [{ url, canGoBack }, scene.key]);
  }
}

function iOSLoading({ navigationType, loading }) {
  return (Platform.OS === 'ios' &&
    !isUndefined(navigationType) &&
    navigationType !== navigationTypes.backForward &&
    navigationType !== navigationTypes.reload &&
    loading === false);
}

function androidLoading(pageLoading, { loading, navigationType }) {
  return (Platform.OS === 'android' &&
    pageLoading &&
    !onLoadingStart &&
    loading &&
    initialized &&
    navigationType !== navigationTypes.backForward &&
    navigationType !== navigationTypes.reload);
}

function createNavigateAction(scene, router) {
  const { webViewWidth, webViewHeight, webViewX, webViewY } = get(scene.route, 'params', {});

  // find a route with webView == true option in config and navigate to it
  const { routeConfigs } = router;
  const routeName = Object.keys(routeConfigs).reduce((name, key) => {
    if (routeConfigs[key].webView === true) {
      return key;
    }
    return name;
  }, 'WebView');

  return NavigationActions.navigate({
    routeName,
    params: {
      webView: true,
      loading: true,
      ...(!isUndefined(webViewWidth) ? { webViewWidth } : {}),
      ...(!isUndefined(webViewHeight) ? { webViewHeight } : {}),
      ...(!isUndefined(webViewX) ? { webViewX } : {}),
      ...(!isUndefined(webViewY) ? { webViewY } : {}),
    },
  });
}

export default onNavigationStateChange;
